/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * (C) 2005-2006,
 * @author JBoss Inc.
 */
/*
 * Copyright (C) 2004,
 *
 * Arjuna Technologies Ltd,
 * Newcastle upon Tyne,
 * Tyne and Wear,
 * UK.
 *
 * $Id: AllObjUidsTest.java 2342 2006-03-30 13:06:17Z  $
 */

package com.hp.mwtests.ts.arjuna.objectstore;

import com.arjuna.ats.arjuna.ArjunaNames;
import com.arjuna.ats.arjuna.objectstore.ObjectStore;
import com.arjuna.ats.arjuna.state.*;
import com.arjuna.ats.arjuna.common.Environment;
import com.arjuna.ats.arjuna.common.Uid;
import com.arjuna.ats.arjuna.coordinator.TxControl;

public class LogStoreTest2
{
	public static void main (String[] args)
	{
		System.setProperty(Environment.OBJECTSTORE_TYPE, ArjunaNames.Implementation_ObjectStore_ActionLogStore().stringForm());
		System.setProperty(Environment.TRANSACTION_LOG_PURGE_TIME, "10000");
		
		ObjectStore objStore = TxControl.getStore();
		final int numberOfTransactions = 1000;
		final Uid[] ids = new Uid[numberOfTransactions];
		final int fakeData = 0xdeedbaaf;
		final String type = "/StateManager/BasicAction/TwoPhaseCoordinator/AtomicAction/Test";
		
		for (int i = 0; i < numberOfTransactions; i++)
		{
			OutputObjectState dummyState = new OutputObjectState();
			
			try
			{
				dummyState.packInt(fakeData);			
				ids[i] = new Uid();
				objStore.write_committed(ids[i], type, dummyState);
			}
			catch (final Exception ex)
			{
				ex.printStackTrace();
			}
		}
		
		try
		{
			objStore.remove_committed(ids[0], type);
		}
		catch (final Exception ex)
		{
			ex.printStackTrace();
		}
		
		try
		{
			/*
			 * Give the purger thread a chance to run and delete
			 * the entry.
			 */
			
			Thread.sleep(12000);
		}
		catch (final Exception ex)
		{
		}
		
		InputObjectState ios = new InputObjectState();
		boolean passed = false;
		
		try
		{
			if (objStore.allObjUids(type, ios, ObjectStore.OS_UNKNOWN))
			{
				Uid id = new Uid(Uid.nullUid());
				int numberOfEntries = 0;
				
				do
				{
					try
					{
						id.unpack(ios);
					}
					catch (Exception ex)
					{
						id = Uid.nullUid();
					}

					if (id.notEquals(Uid.nullUid()))
					{
						passed = true;
						
						numberOfEntries++;
						
						boolean found = false;
						
						for (int i = 0; i < ids.length; i++)
						{
							if (id.equals(ids[i]))
								found = true;
						}
						
						if (passed && !found)
						{
							passed = false;

							System.err.println("Found unexpected transaction!");
						}
					}
				}
				while (id.notEquals(Uid.nullUid()));
				
				if ((numberOfEntries == ids.length -1) && passed)
				{
					if (objStore.currentState(ids[0], type) != ObjectStore.OS_UNKNOWN)
						passed = false;
					else
					{
						if (objStore.currentState(ids[1], type) != ObjectStore.OS_COMMITTED)
							passed = false;
					}
				}
				else
				{
					passed = false;

					System.err.println("Expected "+ids.length+" and got "+numberOfEntries);
				}
			}
		}
		catch (final Exception ex)
		{
			ex.printStackTrace();
		}
		
		System.err.println("Test "+((passed) ? "passed" : "failed"));
	}
}
